module AuthJwt
  # Generate the initializer
  class InstallGenerator < Rails::Generators::Base
    source_root File.expand_path('../templates', __FILE__)

    # generate a random number
    def  gen_secret
      SecureRandom.hex(16)
    end

    desc 'This generator creates an initializer file for AuthJwt at config/initializers/auth_jwt.rb'
    def create_initializer_file
      template 'initializer.rb', 'config/initializers/auth_jwt.rb'
    end

    desc 'This generator creates public/private keys at config/keys/jwt_public_key.pem and config/keys/jwt_private_key.pem'
    def create_keys_files
      rsa_private = OpenSSL::PKey::RSA.generate(2048)
      create_file 'config/keys/jwt_private_key.pem', rsa_private.to_pem
      create_file 'config/keys/jwt_public_key.pem', rsa_private.public_key.to_pem
    end
  end
end
